@@ -42,6 +42,7 @@
 #include <sys/mman.h>
 #include <netinet/udp.h>
 #include <net/if.h>
+#include <libgen.h>
 
 #include <http-parser/http_parser.h>
 
@@ -514,6 +515,31 @@ int stream_close(ngtcp2_conn *conn, uint32_t flags, int64_t stream_id,
 }
 } // namespace
 
+namespace {
+int stream_reset(ngtcp2_conn *conn, int64_t stream_id, uint64_t final_size,
+                 uint64_t app_error_code, void *user_data,
+                 void *stream_user_data) {
+  auto h = static_cast<Handler *>(user_data);
+  if (h->on_stream_reset(stream_id) != 0) {
+    return NGTCP2_ERR_CALLBACK_FAILURE;
+  }
+  return 0;
+}
+} // namespace
+
+int Handler::on_stream_reset(int64_t stream_id) {
+  if (ngtcp2_is_bidi_stream(stream_id)) {
+    if (auto rv = ngtcp2_conn_shutdown_stream_write(conn_, 0, stream_id, 0);
+        rv != 0) {
+      std::cerr << "ngtcp2_conn_shutdown_stream_write: " << ngtcp2_strerror(rv)
+                << std::endl;
+      return -1;
+    }
+  }
+
+  return 0;
+}
+
 namespace {
 void rand(uint8_t *dest, size_t destlen, const ngtcp2_rand_ctx *rand_ctx) {
   auto dis = std::uniform_int_distribution<uint8_t>();
@@ -677,7 +703,7 @@ int Handler::init(const Endpoint &ep, const Address &local_addr,
       ::update_key,
       path_validation,
       nullptr, // select_preferred_addr
-      nullptr, // stream_reset
+      ::stream_reset,
       nullptr, // extend_max_remote_streams_bidi
       nullptr, // extend_max_remote_streams_uni
       ::extend_max_stream_data,
@@ -1902,9 +1928,7 @@ void Server::read_pkt(Endpoint &ep, const Address &local_addr,
       associate_cid(&scids[i], h.get());
     }
 
-    handlers_.emplace(dcid_key, h.get());
-
-    h.release();
+    handlers_.emplace(dcid_key, h.release());
 
     return;
   }
@@ -2438,9 +2462,14 @@ int parse_host_port(Address &dest, int af, const char *first,
 }
 } // namespace
 
+namespace {
+const char *prog = "h09server";
+} // namespace
+
 namespace {
 void print_usage() {
-  std::cerr << "Usage: server [OPTIONS] <ADDR> <PORT> <PRIVATE_KEY_FILE> "
+  std::cerr << "Usage: " << prog
+            << " [OPTIONS] <ADDR> <PORT> <PRIVATE_KEY_FILE> "
                "<CERTIFICATE_FILE>"
             << std::endl;
 }
@@ -2579,7 +2608,7 @@ Options:
               The maximum length of a dynamically generated content.
               Default: )"
             << util::format_uint_iec(config.max_dyn_length) << R"(
-  --cc=(cubic|reno|bbr|bbrv2)
+  --cc=(cubic|reno|bbr)
               The name of congestion controller algorithm.
               Default: )"
             << util::strccalgo(config.cc_algo) << R"(
@@ -2662,6 +2691,10 @@ std::ofstream keylog_file;
 int main(int argc, char **argv) {
   config_set_default(config);
 
+  if (argc) {
+    prog = basename(argv[0]);
+  }
+
   for (;;) {
     static int flag = 0;
     constexpr static option long_opts[] = {
@@ -2889,11 +2922,7 @@ int main(int argc, char **argv) {
           config.cc_algo = NGTCP2_CC_ALGO_BBR;
           break;
         }
-        if (strcmp("bbrv2", optarg) == 0) {
-          config.cc_algo = NGTCP2_CC_ALGO_BBR_V2;
-          break;
-        }
-        std::cerr << "cc: specify cubic, reno, bbr, or bbrv2" << std::endl;
+        std::cerr << "cc: specify cubic, reno, or bbr" << std::endl;
         exit(EXIT_FAILURE);
       case 20:
         // --initial-rtt
